<?php // $Id$

function votesmart_factory_admin_content(&$form_state) {
}

function votesmart_factory_admin_content_submit(&$form_state) {
}

function votesmart_factory_admin_election(&$form_state) {
  $states  = votesmart_get_list('states');
  $offices = votesmart_get_list('office_types');

  $year = date('Y');
  $years = array();
  foreach (range($year - 3, $year + 2) as $y) $years[$y] = $y;
  $form['year'] = array(
    '#type' => 'select',
    '#multiple' => FALSE,
    '#title' => t('Election year'),
    '#options' => $years,
    '#default_value' => $year,
  );

  $form['offices'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Offices'),
    '#options' => $offices,
  );

  $form['states'] = array(
    '#type' => 'checkboxes',
    '#title' => t('States'),
    '#options' => array_merge(array('NA' => 'National elections'), $states),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Begin import')
  );

  return $form;
}

function votesmart_factory_admin_election_submit($form, &$form_state) {
  $year    = $form_state['values']['year'];
  $states  = array_filter($form_state['values']['states']);
  $offices = array_filter($form_state['values']['offices']);

  $batch  = array(
    'title' => t('Importing election data'),
    'operations' => array(
      array('votesmart_factory_admin_election_import', array($year, $offices, $states)),
    ),
    'finished' => 'votesmart_factory_admin_election_finished',
    'file' => drupal_get_path('module', 'votesmart_factory') .'/includes/votesmart_factory.admin.inc',
  );

  batch_set($batch);
  batch_process();
}

function votesmart_factory_admin_election_import($year, $offices, $states, &$context) {
  if (empty($context['sandbox'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = 0;
    $context['sandbox']['current_state'] = current($states);
    $context['sandbox']['elections'] = array();

    // Generate a list of elections we'll be importing content for.
    foreach ($states as $state) {
      $query = array('stateId' => $state, 'year' => $year);
      foreach (votesmart_api('Election.getElectionByYearState', $query, 'election') as $election) {
        $election = (array) $election;
        if (in_array($election['officeTypeId'], $offices)) {
          $context['sandbox']['elections'][] = (array) $election; 
        }
      }
      $context['sandbox']['max'] = count($context['sandbox']['elections']);
    }
    // Return at this point to avoid running too many queries in a batch.
    if ($context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
    return;
  }

  // Get all candidates from The next election in our queue.
  if (!$context['sandbox']['candidates']) {
    $election = $context['sandbox']['election'] = array_shift($context['sandbox']['elections']);
    if ($id = $election['electionId']) {
      $query = array('electionId' => $id);
      $context['sandbox']['candidates'] = votesmart_api('Candidates.getByElection', $query, 'candidate');
    }
  }

  $election = $context['sandbox']['election'];
  $state    = $election['stateId'];

  module_load_include('inc', 'node', 'node.pages');
  $count = 0;
  while ($c = array_shift($context['sandbox']['candidates'])) {
    $c = (array) $c;

    // TODO handle this deduping more cleanly?
    $table = _content_tablename('candidate', CONTENT_DB_STORAGE_PER_CONTENT_TYPE);
    if ($nid = db_result(db_query("SELECT nid FROM {$table} 
      WHERE field_candidate_id_id = '%s'", $c['candidateId']))) {
      $node = node_load($nid);
    }
    else {
      global $user;
      $node = (object) array(
       'type' => 'candidate',
       'name' => $user->name,
       'uid' => $user->uid,
      );
    }

    $node->title = $c['firstName'] .' '. $c['lastName'];

    $node->field_election_office = array(array('value' => $c['electionOffice'], 'table' => 'candidate'));
    $node->field_candidate_id = array(array('id' => $c['candidateId'], 'table' => 'candidate'));
    $node->field_party = array(array('value' => $c['electionParties']));
    if ($state != 'NA') {
      $node->field_state = array(array('value' => $state));
    }

    // TODO this removes all reference to previous years.  Is that cool?
    $node->field_election_year = array(array('value' => $election['electionYear']));

    switch ($election['officeTypeId']) {
      case 'P':  // Presidential
        break;

      case 'C':  // Congressional ( house / senate ).
        // Congressional candidate has a district, senate doesn't (?)
        if ($district = $c['electionDistrictName']) {
          // TODO this presupposes you've added this named field.
          $node->field_congressional_district = array(array('value' => $district));
        }
        break;

      case 'L':  // Legislative ( state house, state senate, etc. )
        if ($district = $c['electionDistrictName']) {
          // TODO this presupposes you've added this named field.
          if (is_numeric($district)) {
            // TODO is the 61 vs. 61A thing a minnesota deal?
            $node->field_upper_district = array(array('value' => $district));
          }
          else {
            $node->field_lower_district = array(array('value' => $district));
          }
        }
        break;
    }         

    // Re-tease and save the node.
    node_submit($node);
    node_save($node);

    // If this is the last candidate, increment the election counter.
    if (!$context['sandbox']['candidates']) {
      $context['sandbox']['progress']++;
    }

    if (++$count == 20) break;
  }

  cache_clear_all();
  $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
}

function votesmart_factory_admin_election_finished($success, $results, $operations) {
  drupal_set_message("Import completed");
  drupal_goto('<front>');
}

function votesmart_factory() {
  require_once(drupal_get_path('module', 'votesmart') .'/votesmart.inc');
  $votesmart_factory_types = variable_get('votesmart_factory_types', array());

  $form   = array();

  $tables = votesmart_tables();
  $lists  = votesmart_lists();

  //print_r(votesmart_api('Election.getElectionByYearState', array('year' => 2008)));
  //print_r(votesmart_api('Election.getElectionByYearState', array('year' => 2008, 'stateId' =>  'MN')));
  //print_r(votesmart_api('Office.getOfficesByBranchLevel', array('branchId' => 'L', 'levelId' => 'L')));
  //print_r(votesmart_get_record('election', 639));
  //print_r(votesmart_api('Leadership.getPositions', array()));
  //print_r(votesmart_api('Candidates.getByElection', array('electionId' => 639)));
  //print_r(votesmart_api('Candidates.getByElection', array('electionId' => 755)));
  //print_r(votesmart_api('Candidates.getByElection', array('electionId' => 756)));
  //print_r(votesmart_api('CandidateBio.getBio', array('candidateId' => 9490)));


  foreach ($tables as $name => $table) {
    // Incomplete table definition. Ignore.
    if (!$table['fields']) continue;

    $form['tables'][$name] = array(
      '#tree' => TRUE,
    );

    $form['tables'][$name]['selected'] = array(
      '#type' => 'checkbox',
      '#title' => $table['description'],
    );

    $form['tables'][$name]['type_name'] = array(
      '#type' => 'textfield',
      '#title' => t('Type'),
      '#description' => t('The machine-readable name of this content type.'),
      '#default_value' => $name,
    );

    foreach ($table['fields'] as $field_name => $field) {
      $form['tables'][$name]['fields'][$field_name] = array(
        '#type' => 'checkbox',
        '#title' => $field['description'],
      );
    }
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Create content types'));

  return $form;
}

function votesmart_factory_submit($form, &$form_state) {
  foreach (votesmart_tables() as $name => $table) {
    if ($form_state['values'][$name]['selected']) {
      $type_name = $form_state['values'][$name]['type_name'];
      $fields = $form_state['values'][$name]['fields'];
      votesmart_factory_import_node_type($type_name, $name, $fields);
    }
  }

}

function votesmart_factory_import_node_type($type_name, $table_name, $fields) {
  $table = current(votesmart_tables($table_name));
  $votesmart_factory_types = variable_get('votesmart_factory_types', array());

  include_once(drupal_get_path('module', 'content_copy') . '/content_copy.module');

  $content = array(
    'type' => array(
      'name' => $table['description'],
      'type' => $type_name,
      'description' => t('@type information provided by Project Vote Smart', array('@type' => $table['description'])),
      'title_label' => 'Name',
      'has_title' => TRUE,
      'body_label' => 'Description',
      'has_body' => FALSE,
      'help' => '',
      'node_options' => array(
        'status' => true,
        'promote' => false,
        'sticky' => false,
        'revision' => false,
      ),
      'old_type' => $type_name,
      'orig_type' => $type_name,
      'module' => 'node',
      'custom' => '1',
      'modified' => '1',
      'locked' => '0',
      'comment' => '0',
    ),
    'fields' => array(),
  );

  // Field mappings to the Vote Smart API
  $map = array();

  foreach ($fields as $field_name => $enabled) {
    if (!$enabled) continue;
    
    if ($field = _votesmart_factory_field($table, $field_name, $table_name, $content, $map)) {
      $content['fields'][$field_name] = $field;
    }
  }

  // Save the default field mappings to the votesmart field.
  if ($id = $content['votesmart_identifier']) {
    $content['fields'][$id]['votesmart_map'][$type_name] = $map;
  }

  $form_state = array(
    'values' => array(
      'type_name' => '<create>',
      'macro' => '$content = ' . var_export($content, TRUE) . ';',
    ),
  );

  drupal_execute('content_copy_import_form', $form_state);
  content_clear_type_cache();

  // Keep track of our types.
  $votesmart_factory_types[] = $type;
  variable_set('votesmart_factory_types', $votesmart_factory_types);
}

function _votesmart_factory_field($table, $name, $table_name, &$content, &$map) {
  $table_field = $table['fields'][$name];

  // Default field definition.
  $field_name = 'field_'. $name;
  $field = array(
    'label' => $table_field['description'],
    'field_name' => $field_name, // TODO not specific enough.
    'type' => 'text',
    'widget_type' => 'text_textfield',
    'required' => '0',
    'multiple' => '0',
    'module' => 'text',
    'widget_module' => 'text',
  );

  // Default map placement.
  $field_map[$field_name] = array('value' => $name);

  // Set as title.
  if (in_array($name, array('name', 'title'))) {
    $field_map['title'] = $name;
    $content['title_label'] = $table_field['description'];
    unset($field, $field_map[$field_name]);
  }

  // Set as body.
  if (in_array($name, array('body'))) {
    $field_map['body'] = $name;
    $content['body_label'] = $table_field['description'];
    $content['has_body'] = TRUE;
    unset($field, $field_map[$field_name]);
  }

  // Convert to numeric field if we've got the number module.
  if (module_exists('number')) {
    $changed = false;
    if ($table_field['type'] == 'integer') {
      $changed = $field['type'] = 'number_integer';
    }
    elseif ($table_field['type'] == 'float') {
      $changed = $field['type'] = 'number_float';
    }
    if ($changed) {
      $field['widget_type'] = 'number';
      $field['module'] = $field['widget_module'] = 'number';
    }

    // Convert boolean items to checkboxes.
    if ($table_field['callback'] == '_votesmart_boolean' && module_exists('optionwidgets')) {
      $field['widget_type'] = 'optionwidgets_onoff';
      $field['widget_module'] = 'optionwidgets';
      $field['allowed_values'] = "0|False\n1|". $field['label'];
    }
  }

  // Convert "exploded" values to multi-value fields.
  if ($table_field['callback'] == '_votesmart_split_newline') {
    $field['multiple'] = '1';
  }

  // Add input formatting to long text fields.
  if ($name != 'body' && ($table_field['type'] == 'long' || $table_field['type'] == 'text')) {
    $field['widget_type'] = 'text_textarea';
    $field['text_processing'] = 1;
  }

  // Convert to image field if we've got the imagefield module.
  if ($name == 'photo' && module_exists('imagefield')) {
  }

  // Convert to link field if we've got the link module.
  if (($name == 'url' || substr($name, -4) == '_url') && module_exists('link')) {
    $field['type'] = $field['widget_type'] = $field['widget_module'] = 'link'; 
    $field_map[$field_name] = array('url' => $name, 'title' => $table_field['description']);
  }

  // Convert to geo reference field if we've got the geo_data module and 
  // appropriate tables.
  if ($geo_type = $table_field['geo_type'] && module_exists('geo_data')) {
    $field['type'] = 'geo_data';
    $field['widget_type'] = 'geo_data_select';
    $field['module'] = $field['widget_module'] = 'geo_data';
    drupal_set_message(t("Adding a geo_data field for @name.  You will need to go to the field settings page to link this field to the appropriate database.", array('@name' => $table_field['description'])), 'warning');
  }

  // Convert multiple fields to postal field if we've got the postal module.

  // Convert to a votesmart field if it's an identifier.
  if ($name == $table_name .'_id') {
    $field['type'] = 'votesmart_id';
    $field['widget_type'] = 'votesmart_id';
    $field['module'] = $field['widget_module'] = 'votesmart_field';
    $field['votesmart_table'] = $table_name;
    $content['votesmart_identifier'] = $name;
  }

  if ($field_map) {
    $map = array_merge($map, $field_map);
  }

  return $field;
}
