<?php // $Id$

function votesmart_settings() {
  $form = array();
  $form['api'] = array(
    '#type' => 'fieldset',
    '#title' => ('API Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => variable_get('votesmart_key', ''),
  );
  $form['api']['votesmart_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Votesmart API server'),
    '#description' => t(''),
    '#default_value' => variable_get('votesmart_host', 'http://api.votesmart.org/'),
  );
  $form['api']['votesmart_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Votesmart API key'),
    '#description' => t(''),
    '#default_value' => variable_get('votesmart_key', ''),
  );
  
  return system_settings_form($form);
}

function _votesmart_field_values($table, $data) {
  $values = array();
  // Make sure this is a valid table array.
  if (!isset($table['votesmart'])) $table = current(votesmart_tables($table));

  foreach ($table['fields'] as $name => $info) {
    if ($key = $info['votesmart']) {
      if (isset($data[$key])) {
        $values[$name] = (string) $data[$key];
        if ($func = $info['callback']) $values[$name] = $func($values[$name]);
      }
    }
  }
  return $values;
}

function _votesmart_set_query($request, $input = array()) {
  $query = array();
  foreach($request[1] as $name => $token) {
    if (isset($input[$token])) {
      $query[$name] = $input[$token];
    }
  }
  $request[1] = $query;
  return $request;
}

function _votesmart_lists($filter = array()) {
  if ($filter && !is_array($filter)) $filter = array($filter);
  $lists = array();

  $lists['states'] = array(
    'description' => t('State'),
    'table' => 'state',
    'votesmart' => array('State.getStateIDs', array(), 'list.state'),
    'id' => 'stateId',
    'name' => 'name',
  );

  $lists['counties'] = array(
    'description' => t('Counties'),
    'votesmart' => array('Local.getCounties', array('StateId' => 'state_id'), ''),
  );

  $lists['elections'] = array(
    'description' => t('Elections by state'),
    'votesmart' => array('Election.getElectionByYearState', array('stateId' => 'state_id', 'year' => 'year'), 'elections.election'),
  );

  $lists['office_types'] = array(
    'description' => t('Office type'),
    'votesmart' => array('Office.getTypes', array(), 'type'),
    'id' => 'officeTypeId',
    'name' => 'name',
  );

  $lists['officials'] = array(
    'description' => t('Officials'),
    'votesmart' => array('Local.getOfficials', array('localId' => 'local_id'), ''),
  );

  $lists['office_branches'] = array(
    'description' => t('Office branch'),
    'votesmart' => array('Office.getBranches', array(), 'branch'),
  );

  $lists['office_levels'] = array(
    'description' => t('Office level'),
    'votesmart' => array('Office.getLevels', array(), 'level'),
  );

  $lists['sig_categories'] = array(
    'description' => t('Special interest Category'),
    'votesmart' => array('Rating.getCategories', array('stateId' => 'state_id'), 'category'),
  );

  $lists['sigs'] = array(
    'description' => t('Special interest group'),
    'table' => 'sig',
    'votesmart' => array('Rating.getSigList', array('categoryId' => 'category_id', 'stateId' => 'state_id'), 'sig'),
  );

  $lists['bill_categories'] = array(
    'description' => t('Bill Categories'),
    'votesmart' => array('Votes.getCategories', array('stateId' => 'state_id', 'year' => 'year'), ''),
  );

  $lists['bills_by_year_state'] = array(
    'description' => t('Bills by year'),
    'table' => 'bill',
    'votesmart' => array('Votes.getBillsByYearState', array('stateId' => 'state_id', 'year' => 'year'), 'bill'),
  );

  $lists['bills_by_category_year_state'] = array(
    'description' => t('Votes by year, category and state'),
    'table' => 'bill',
    'votesmart' => array('Votes.getBillsByCategoryYearState', array('stateId' => 'state_id', 'year' => 'year', 'categoryId' => 'category_id'), 'bill'),
  );

  $lists['bills_by_official_year_office'] = array(
    'description' => t('Votes by official, year and office'),
    'table' => 'bill',
    'votesmart' => array('Votes.getBillsByOfficialYearOffice', array('candidateId' => 'candidate_id', 'year' => 'year', 'office' => 'office_id'), 'bill'),
  );

  $lists['bills_by_candidate_category_office'] = array(
    'description' => t('Votes by official, year and office'),
    'table' => 'bill',
    'votesmart' => array('Votes.getBillsByOfficialYearOffice', array('candidateId' => 'candidate_id', 'year' => 'year', 'office' => 'office_id'), 'bill'),
  );

  if ($filter) {
    $ret = array();
    foreach ($filter as $list) {
      $ret[$list] = $lists[$list];
    }
    return $ret;
  }

  return $lists;
}

function _votesmart_tables($filter = array()) {
  if ($filter && !is_array($filter)) $filter = array($filter);
  $tables = array();
  $tables['address'] = array(
  );
  $tables['measure'] = array(
    'description' => t('Measure'),
    'votesmart' => array('Measure.getMeasure', array('measureId' => 'measure_id'), 'measure'),
    'fields' => array(
      'measure_id' => array(
        'type' => 'integer',
        'description' => t('Measure'),
        'votesmart' => 'measureId',
      ),
      'measure_code' => array(
        'type' => 'varchar',
        'length' => 100,
        'description' => t('Measure code'),
        'votesmart' => 'measureCode',
      ),
      'title' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Title'),
        'votesmart' => 'title',
      ),
      'election_date' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Election date'),
        'votesmart' => 'electionDate',
      ),
      'election_type' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Election type'),
        'votesmart' => 'electionType',
      ),
      'source' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Source'),
        'votesmart' => 'source',
      ),
      'url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('URL'),
        'votesmart' => 'url',
      ),
      'summary' => array(
        'type' => 'long',
        'description' => t('Summary'),
        'votesmart' => 'summary',
      ),
      'summary_url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Summary URL'),
        'votesmart' => 'summaryUrl',
      ),
      'body' => array(
        'type' => 'long',
        'description' => t('Measure text'),
        'votesmart' => 'measureText',
      ),
      'text_url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Text URL'),
        'votesmart' => 'textUrl',
      ),
      'pro_url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Pro URL'),
        'votesmart' => 'proUrl',
      ),
      'con_url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Con URL'),
        'votesmart' => 'conUrl',
      ),
      'yes' => array(
        'type' => 'integer',
        'description' => t('Yes'),
        'votesmart' => 'yes',
      ),
      'no' => array(
        'type' => 'integer',
        'description' => t('No'),
        'votesmart' => 'no',
      ),
      'outcome' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Outcome'),
        'votesmart' => 'outcome',
      ),
    ),
  );
  $tables['bill'] = array(
    'description' => t('Bill'),
    'votesmart' => array('Votes.getBill', array('billId' => 'bill_id'), 'bill'),
  );
  $tables['bill_vote'] = array(
    'description' => t('Bill Action'),
    'votesmart' => array('Votes.getBillActionVotes', array('actionId' => 'action_id'), 'vote'),
  );
  $tables['candidate'] = array(
    'description' => t('Candidate'),
    'votesmart' => array('CandidateBio.getBio', array('candidateId' => 'candidate_id'), 'candidate'),
    'fields' => array(
      'candidate_id' => array(
        'type' => 'integer',
        'description' => t('Candidate'),
        'votesmart' => 'candidateId',
      ),
      'fecid' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Federal Election Comission ID'),
        'votesmart' => 'fecid',
      ),
      'first_name' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('First name'),
        'votesmart' => 'firstName',
      ),
      'middle_name' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Middle name'),
        'votesmart' => 'middleName',
      ),
      'last_name' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Last name'),
        'votesmart' => 'lastName',
      ),
      'suffix' => array(
        'type' => 'varchar',
        'size' => 10,
        'description' => t('Suffix'),
        'votesmart' => 'suffix',
      ),
      'birth_date' => array(
        'type' => 'date',
        'description' => t('Birth date'),
        'votesmart' => 'birthDate',
      ),
      'birth_place' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Birth place'),
        'votesmart' => 'birthPlace',
      ),
      'pronounciation' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Pronounciation'),
        'votesmart' => 'pronounciation',
      ),
      'gender' => array(
        'type' => 'varchar',
        'size' => 6,
        'description' => t('Gender'),
        'votesmart' => 'gender',
      ),
      'family' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Family'),
        'votesmart' => 'family',
        'callback' => '_votesmart_split_newline',
      ),
      'photo' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Photo'),
        'votesmart' => 'photo',
      ),
      'home_city' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Home city'),
        'votesmart' => 'homeCity',
      ),
      'home_state' => array(
        'type' => 'varchar',
        'size' => 2,
        'description' => t('Home state'),
        'votesmart' => 'homeState',
      ),
      'education' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Education'),
        'votesmart' => 'education',
        'callback' => '_votesmart_split_newline',
      ),
      'profession' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Profession'),
        'votesmart' => 'profession',
        'callback' => '_votesmart_split_newline',
      ),
      'political' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Political'),
        'votesmart' => 'political',
        'callback' => '_votesmart_split_newline',
      ),
      'religion' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Religion'),
        'votesmart' => 'religion',
      ),
      'cong_membership' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Congressional membership'),
        'votesmart' => 'congMembership',
        'callback' => '_votesmart_split_newline',
      ),
      'org_membership' => array(
        'type' => 'varchar',
        'size' => 100,
        'description' => t('Organizational membership'),
        'votesmart' => 'orgMembership',
        'callback' => '_votesmart_split_newline',
      ),
      'special_message' => array(
        'type' => 'text',
        'description' => t('Special message'),
        'votesmart' => 'specialMsg',
      ),

      // These fields are not defined in the candidate record but in other
      // queries, such as elections, etc.  Perhaps we move them, or otherwise
      // Indicate the relationship.  For now, you're expected to populate them
      // manually ( without tokens ), and/or through an import script.
      'election_year' => array(
        'type' => 'int',
        'description' => t('Election Year'),
        'callback' => '_votesmart_split_newline',
      ),
      'state' => array(
        'type' => 'char',
        'size' => 2,
        'description' => t('State'),
        'geo_type' => 'polygon',
        'callback' => '_votesmart_split_newline',
      ),
      'party' => array(
        'type' => 'int',
        'description' => t('Party'),
        'callback' => '_votesmart_split_newline',
      ),
      'congressional_district' => array(
        'type' => 'int',
        'geo_type' => 'polygon',
        'description' => t('Congressional district'),
      ),
      'upper_district' => array(
        'type' => 'varchar',
        'size' => '5',
        'geo_type' => 'polygon',
        'description' => t('State senate district'),
      ),
      'lower_district' => array(
        'type' => 'varchar',
        'size' => '5',
        'geo_type' => 'polygon',
        'description' => t('State house district'),
      ),
    ),
  );
  $tables['committee'] = array(
  );
  $tables['district'] = array(
  );
  $tables['election'] = array(
    'description' => t('Election'),
    'votesmart' => array('Election.getElection', array('electionId' => 'election_id')),
    'fields' => array(
      'election_id' => array(
        'type' => 'integer',
        'description' => t('Election'),
        'votesmart' => 'electionId',
      ),
      'name' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Election name'),
        'votesmart' => 'name',
      ),
      'state' => array(
        'type' => 'varchar',
        'length' => 2,
        'description' => t('State'),
        'votesmart' => 'stateId',
      ),
      'office_type' => array(
        'type' => 'integer',
        'description' => t('Office type'),
        'votesmart' => 'officeTypeId',
        'votesmart_list' => 'office_types',
      ),
      'special' => array(
        'type' => 'integer',
        'description' => t('Special election'),
        'votesmart' => 'special',
        'callback' => '_votesmart_boolean',
      ),
      'year' => array(
        'type' => 'integer',
        'description' => t('Election year'),
        'votesmart' => 'electionYear',
      ),
    ),
  );
  $tables['position'] = array(
  );
  $tables['city'] = array(
  );
  $tables['county'] = array(
  );
  $tables['npat'] = array(
  );
  $tables['office'] = array(
  );
  $tables['officials'] = array(
  );
  $tables['state'] = array(
    'description' => t('State'),
    'votesmart' => array('State.getState', array('stateId' => 'state_id'), 'details'),
    'fields' => array(
      'state_id' => array(
        'type' => 'char',
        'length' => 2, 
        'description' => t('State code'),
        'votesmart' => 'stateId',
      ),
      'type' => array(
        'type' => 'varchar',
        'length' => 10, 
        'description' => t('State type'),
        'votesmart' => 'stateType'
      ),
      'name' => array(
        'type' => 'varchar',
        'length' => 100, 
        'description' => t('State'),
        'votesmart' => 'name'
      ),
      'nickname' => array(
        'type' => 'varchar',
        'length' => 100, 
        'description' => t('Nickname'),
        'votesmart' => 'nickName'
      ),
      'capital' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Capital'),
        'votesmart' => 'capital'
      ),
      'area' => array(
        'type' => 'int',
        'description' => t('Area'),
        'votesmart' => 'area',
        'callback' => '_votesmart_numeric',
      ),
      'population' => array(
        'type' => 'int',
        'description' => t('Population'),
        'votesmart' => 'population',
        'callback' => '_votesmart_numeric',
      ),
      'statehood' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Statehood'),
        'votesmart' => 'statehood',
        'callback' => '_votesmart_date',
      ),
      'motto' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Motto'),
        'votesmart' => 'motto'
      ),
      'flower' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Flower'),
        'votesmart' => 'flower'
      ),
      'tree' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Tree'),
        'votesmart' => 'tree'
      ),
      'bird' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Bird'),
        'votesmart' => 'bird'
      ),
      'highpoint' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Highest point'),
        'votesmart' => 'highPoint'
      ),
      'lowpoint' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Lowest point'),
        'votesmart' => 'lowPoint'
      ),
      'bicameral' => array(
        'type' => 'varchar',
        'description' => t('Bicameral legislature'),
        'votesmart' => 'bicameral',
        'callback' => '_votesmart_boolean',
      ),
      'upper_legislature' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Upper legislature'),
        'votesmart' => 'upperLegis'
      ),
      'lower_legislature' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Lower legislature'),
        'votesmart' => 'lowerLegis'
      ),
      'leutenant_gov' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Leutenant governor'),
        'votesmart' => 'ltGov',
        'callback' => '_votesmart_boolean',
      ),
      'senators' => array(
        'type' => 'integer',
        'description' => t('Senators'),
        'votesmart' => 'senators'
      ),
      'reps' => array(
        'type' => 'integer',
        'description' => t('Representatives'),
        'votesmart' => 'reps'
      ),
      'term_limit' => array(
        'type' => 'integer',
        'description' => t('Term limit'),
        'votesmart' => 'termLimit'
      ),
      'term_length' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Term length'),
        'votesmart' => 'termLength'
      ),
      'bill_url' => array(
        'type' => 'varchar',
        'length' => 255, 
        'description' => t('Bill URL'),
        'votesmart' => 'billUrl'
      ),
      'vote_url' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Vote URL'),
        'votesmart' => 'voteUrl'
      ),
      'voter_registration' => array(
        'type' => 'long',
        'description' => t('Voter registration'),
        'votesmart' => 'voterReg'
      ),
      'primary_date' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('Primary election'),
        'votesmart' => 'primaryDate'
      ),
      'general_date' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('General election'),
        'votesmart' => 'generalDate'
      ),
      'absentee_who' => array(
        'type' => 'varchar',
        'length' => 200, 
        'votesmart' => 'absenteeWho'
      ),
      'absentee_how' => array(
        'type' => 'varchar',
        'length' => 200, 
        'votesmart' => 'absenteeHow'
      ),
      'absentee_when' => array(
        'type' => 'varchar',
        'length' => 200, 
        'votesmart' => 'absenteeWhen'
      ),
      'largest_city' => array(
        'type' => 'varchar',
        'length' => 255, 
        'description' => t('Largest city'),
        'votesmart' => 'largestCity'
      ),
      'roll_upper' => array(
        'type' => 'varchar',
        'length' => 100, 
        'votesmart' => 'rollUpper'
      ),
      'roll_lower' => array(
        'type' => 'varchar',
        'length' => 200, 
        'votesmart' => 'rollLower'
      ),
      'us_circuit' => array(
        'type' => 'varchar',
        'length' => 200, 
        'description' => t('U.S. circuit'),
        'votesmart' => 'usCircuit'
      ),
    ),
  );
  $tables['sig'] = array(
    'description' => t('Special interest group'),
		'votesmart' => array('Rating.getSig', array('sigId' => 'sig_id')),
    'fields' => array(
      'sig_id' => array(
        'type' => 'integer',
        'description' => t('Special interest group id'),
        'votesmart' => 'sigId'
      ),
      'parent_id' => array(
        'type' => 'integer',
        'description' => t('Special interest group parent'),
        'votesmart' => 'parentId'
      ),
      'state_id' => array(
        'type' => 'char',
        'length' => 2,
        'description' => t('State'),
        'votesmart' => 'stateId',
      ),
      'name' => array(
        'type' => 'name',
        'length' => 200,
        'description' => t('Name'),
        'votesmart' => 'name',
      ),
      'description' => array(
        'type' => 'long',
        'description' => t('Description'),
        'votesmart' => 'description',
      ),
      'address' => array(
        'type' => 'varchar',
        'length' => 255,
        'description' => t('Address'),
        'votesmart' => 'address',
      ),
      'city' => array(
        'type' => 'varchar',
        'length' => 255,
        'description' => t('City'),
        'votesmart' => 'city',
      ),
      'state' => array(
        'type' => 'varchar',
        'length' => 2,
        'description' => t('State'),
        'votesmart' => 'state',
      ),
      'zip' => array(
        'type' => 'varchar',
        'length' => 10,
        'description' => t('Zip'),
        'votesmart' => 'zip',
      ),
      'phone1' => array(
        'type' => 'varchar',
        'length' => 10,
        'description' => t('Phone number'),
        'votesmart' => 'phone1',
      ),
      'phone2' => array(
        'type' => 'varchar',
        'length' => 10,
        'description' => t('Additional phone number'),
        'votesmart' => 'phone2',
      ),
      'fax' => array(
        'type' => 'varchar',
        'length' => 10,
        'description' => t('Fax'),
        'votesmart' => 'fax',
      ),
      'email' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('E-mail'),
        'votesmart' => 'email',
      ),
      'url' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('URL'),
        'votesmart' => 'url',
      ),
      'contact_name' => array(
        'type' => 'varchar',
        'length' => 200,
        'description' => t('Contact name'),
        'votesmart' => 'contactName',
      ),
    ),
  );
  $tables['rating'] = array(
  );
  $tables['vote'] = array(
  );

  if ($filter) {
    $ret = array();
    foreach ($filter as $table) {
      $ret[$table] = $tables[$table];
    }
    return $ret;
  }

  return $tables;
}

/**
 * Basic value conversion functions.
 */
function _votesmart_numeric($value) {
  return (preg_replace('/[^\d.]/', '', $value));
}

function _votesmart_boolean($value) {
  return ($value == 't');
}

function _votesmart_date($value) {
}

function _votesmart_split_newline($value) {
  return (explode("\n", $value));
}
